@@ -10,7 +10,7 @@
 #   IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 #   WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
-#   $Id: epchar.c,v 1.14 2001/04/27 06:37:58 richter Exp $
+#   $Id: epchar.c,v 1.16 2001/09/13 07:28:47 richter Exp $
 #
 ###################################################################################*/
 
@@ -202,7 +202,7 @@ struct tCharTrans Char2Html [] =
         { '' ,   "&copy;"  },    /* 	Copyright               */
         { '' ,   "&ordf;"  },    /* 	Feminine ordinal  */
         { '' ,   "&laquo;" },    /* 	Left angle quote, guillemot left  */
-        { '' ,   "&not;"   },    /*	Not sign  */
+        { '' ,   "&euro;"   },    /*	Euro sign  */
         { '' ,   "&shy;"    },    /* 	Soft hyphen  */
         { '' ,   "&reg;"    },    /* 	Registered trademark  */
 /*        { '' ,   "&macr;"   },    / *  	Macron accent  */
@@ -328,7 +328,7 @@ struct tCharTrans Char2Url [] =
         { ' ' ,   "%20"           },    /* 	&#32;		Space  */
         { '!' ,   ""         },    /* 	&#33;		Exclamation mark  */
         { '"' ,   "%22"   },    /* 	Quotation mark  */
-        { '#' ,   ""         },    /* 	&#35;		Number sign  */
+        { '#' ,   "%23"      },    /* 	&#35;		Number sign  */
         { '$' ,   ""         },    /* 	&#36;		Dollar sign  */
         { '%' ,   "%25"      },    /* 	&#37;		Percent sign  */
         { '&' ,   "%26"    },    /* 	Ampersand  */
@@ -642,6 +642,7 @@ struct tCharTrans Html2Char [] =
         { '' ,   "&egrave" },    /* 	Small e, grave accent  */
         { '' ,   "&eth"    },    /* 	Small eth, Icelandic  */
         { '' ,   "&euml"   },    /* 	Small e, diresis / umlaut  */
+        { '' ,   "&euro"   },    /*	Euro sign  */
         { '' ,   "&frac12" },    /* 	Fraction one-half  */
         { '' ,   "&frac14" },    /* 	Fraction one-fourth  */
         { '' ,   "&frac34" },    /* 	Fraction three-fourths  */
@@ -659,7 +660,6 @@ struct tCharTrans Html2Char [] =
         { '' ,   "&micro"  },    /* 	Micro sign  */
         { '' ,   "&middot" },    /* 	Middle dot  */
         { ' ' ,   "&nbsp"   },    /* 	Non-breaking Space  */
-        { '' ,   "&not"   },    /*	Not sign  */
         { '' ,   "&ntilde" },    /* 	Small n, tilde  */
         { '' ,   "&oacute" },    /* 	Small o, acute accent  */
         { '' ,   "&ocirc"  },    /* 	Small o, circumflex  */
