@@ -10,7 +10,7 @@
 #   IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 #   WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
-#   $Id: epeval.c,v 1.29 2001/05/29 06:28:10 richter Exp $
+#   $Id: epeval.c,v 1.30 2001/11/02 10:03:48 richter Exp $
 #
 ###################################################################################*/
 
@@ -540,9 +540,6 @@ int CallCV  (/*i/o*/ register req * r,
 	return rcEvalErr ;
         }
 
-     if (r -> bDebug & dbgWatchScalar)
-         Watch (r) ;
-
      
     return ok ;
     }
